@@ -6,9 +6,6 @@ Carrot compatible interface for :class:`Publisher` and :class:`Producer`.
 
 See http://packages.python.org/pypi/carrot for documentation.
 
-:copyright: (c) 2009 - 2012 by Ask Solem.
-:license: BSD, see LICENSE for more details.
-
 """
 from __future__ import absolute_import
 
@@ -19,6 +16,9 @@ from .entity import Exchange, Queue
 
 __all__ = ['Publisher', 'Consumer']
 
+# XXX compat attribute
+entry_to_queue = Queue.from_dict
+
 
 def _iterconsume(connection, consumer, no_ack=False, limit=None):
     consumer.consume(no_ack=no_ack)
