@@ -1,15 +1,20 @@
 from __future__ import absolute_import
 from __future__ import with_statement
 
+import errno
 import pickle
+import socket
 
+from copy import copy
+from mock import patch
 from nose import SkipTest
 
-from kombu.connection import BrokerConnection, Resource, parse_url
-from kombu.messaging import Consumer, Producer
+from kombu import Connection, Consumer, Producer, parse_url
+from kombu.connection import Resource
 
 from .mocks import Transport
 from .utils import TestCase
+
 from .utils import Mock, skip_if_not_module
 
 
@@ -36,7 +41,7 @@ class test_connection_utils(TestCase):
         self.assertEqual(result['hostname'], 'example.com/')
 
     def test_parse_generated_as_uri(self):
-        conn = BrokerConnection(self.url)
+        conn = Connection(self.url)
         info = conn.info()
         for k, v in self.expected.items():
             self.assertEqual(info[k], v)
@@ -46,12 +51,12 @@ class test_connection_utils(TestCase):
 
     @skip_if_not_module('pymongo')
     def test_as_uri_when_mongodb(self):
-        x = BrokerConnection('mongodb://localhost')
+        x = Connection('mongodb://localhost')
         self.assertTrue(x.as_uri())
 
     def test_bogus_scheme(self):
         with self.assertRaises(KeyError):
-            BrokerConnection('bogus://localhost:7421').transport
+            Connection('bogus://localhost:7421').transport
 
     def assert_info(self, conn, **fields):
         info = conn.info()
@@ -60,79 +65,77 @@ class test_connection_utils(TestCase):
 
     def test_rabbitmq_example_urls(self):
         # see Appendix A of http://www.rabbitmq.com/uri-spec.html
-        C = BrokerConnection
 
         self.assert_info(
-            C('amqp://user:pass@host:10000/vhost'),
+            Connection('amqp://user:pass@host:10000/vhost'),
                 userid='user', password='pass', hostname='host',
                 port=10000, virtual_host='vhost')
 
         self.assert_info(
-            C('amqp://user%61:%61pass@ho%61st:10000/v%2fhost'),
+            Connection('amqp://user%61:%61pass@ho%61st:10000/v%2fhost'),
                 userid='usera', password='apass',
                 hostname='hoast', port=10000,
                 virtual_host='v/host')
 
         self.assert_info(
-            C('amqp://'),
+            Connection('amqp://'),
                 userid='guest', password='guest',
                 hostname='localhost', port=5672,
                 virtual_host='/')
 
         self.assert_info(
-            C('amqp://:@/'),
+            Connection('amqp://:@/'),
                 userid='guest', password='guest',
                 hostname='localhost', port=5672,
                 virtual_host='/')
 
         self.assert_info(
-            C('amqp://user@/'),
+            Connection('amqp://user@/'),
                 userid='user', password='guest',
                 hostname='localhost', port=5672,
                 virtual_host='/')
 
         self.assert_info(
-            C('amqp://user:pass@/'),
+            Connection('amqp://user:pass@/'),
                 userid='user', password='pass',
                 hostname='localhost', port=5672,
                 virtual_host='/')
 
         self.assert_info(
-            C('amqp://host'),
+            Connection('amqp://host'),
                 userid='guest', password='guest',
                 hostname='host', port=5672,
                 virtual_host='/')
 
         self.assert_info(
-            C('amqp://:10000'),
+            Connection('amqp://:10000'),
                 userid='guest', password='guest',
                 hostname='localhost', port=10000,
                 virtual_host='/')
 
         self.assert_info(
-            C('amqp:///vhost'),
+            Connection('amqp:///vhost'),
                 userid='guest', password='guest',
                 hostname='localhost', port=5672,
                 virtual_host='vhost')
 
         self.assert_info(
-            C('amqp://host/'),
+            Connection('amqp://host/'),
                 userid='guest', password='guest',
                 hostname='host', port=5672,
                 virtual_host='/')
 
         self.assert_info(
-            C('amqp://host/%2f'),
+            Connection('amqp://host/%2f'),
                 userid='guest', password='guest',
                 hostname='host', port=5672,
                 virtual_host='/')
 
     def test_url_IPV6(self):
-        C = BrokerConnection
         raise SkipTest("urllib can't parse ipv6 urls")
 
         self.assert_info(
-            C('amqp://[::1]'),
+            Connection('amqp://[::1]'),
                 userid='guest', password='guest',
                 hostname='[::1]', port=5672,
                 virtual_host='/')
@@ -141,7 +144,7 @@ class test_connection_utils(TestCase):
 class test_Connection(TestCase):
 
     def setUp(self):
-        self.conn = BrokerConnection(port=5672, transport=Transport)
+        self.conn = Connection(port=5672, transport=Transport)
 
     def test_establish_connection(self):
         conn = self.conn
@@ -156,6 +159,157 @@ class test_Connection(TestCase):
         self.assertFalse(_connection.connected)
         self.assertIsInstance(conn.transport, Transport)
 
+    def test_multiple_urls(self):
+        conn1 = Connection('amqp://foo;amqp://bar')
+        self.assertEqual(conn1.hostname, 'foo')
+        self.assertListEqual(conn1.alt, ['amqp://foo', 'amqp://bar'])
+
+        conn2 = Connection(['amqp://foo', 'amqp://bar'])
+        self.assertEqual(conn2.hostname, 'foo')
+        self.assertListEqual(conn2.alt, ['amqp://foo', 'amqp://bar'])
+
+    def test_uri_passthrough(self):
+        from kombu import connection as mod
+        prev, mod.URI_PASSTHROUGH = mod.URI_PASSTHROUGH, set(['foo'])
+        try:
+            with patch('kombu.connection.parse_url') as parse_url:
+                c = Connection('foo+mysql://some_host')
+                self.assertEqual(c.transport_cls, 'foo')
+                self.assertFalse(parse_url.called)
+                self.assertEqual(c.hostname, 'mysql://some_host')
+                self.assertTrue(c.as_uri().startswith('foo+'))
+            with patch('kombu.connection.parse_url') as parse_url:
+                c = Connection('mysql://some_host', transport='foo')
+                self.assertEqual(c.transport_cls, 'foo')
+                self.assertFalse(parse_url.called)
+                self.assertEqual(c.hostname, 'mysql://some_host')
+        finally:
+            mod.URI_PASSTHROUGH = prev
+        c = Connection('amqp+sqlite://some_host')
+        self.assertTrue(c.as_uri().startswith('amqp+'))
+
+    def test_default_ensure_callback(self):
+        with patch('kombu.connection.logger') as logger:
+            c = Connection(transport=Mock)
+            c._default_ensure_callback(KeyError(), 3)
+            self.assertTrue(logger.error.called)
+
+    def test_ensure_connection_on_error(self):
+        c = Connection('amqp://A;amqp://B')
+        with patch('kombu.connection.retry_over_time') as rot:
+            c.ensure_connection()
+            self.assertTrue(rot.called)
+
+            args = rot.call_args[0]
+            cb = args[4]
+            intervals = iter([1, 2, 3, 4, 5])
+            self.assertEqual(cb(KeyError(), intervals, 0), 0)
+            self.assertEqual(cb(KeyError(), intervals, 1), 1)
+            self.assertEqual(cb(KeyError(), intervals, 2), 0)
+            self.assertEqual(cb(KeyError(), intervals, 3), 2)
+            self.assertEqual(cb(KeyError(), intervals, 4), 0)
+            self.assertEqual(cb(KeyError(), intervals, 5), 3)
+            self.assertEqual(cb(KeyError(), intervals, 6), 0)
+            self.assertEqual(cb(KeyError(), intervals, 7), 4)
+
+            errback = Mock()
+            c.ensure_connection(errback=errback)
+            args = rot.call_args[0]
+            cb = args[4]
+            self.assertEqual(cb(KeyError(), intervals, 0), 0)
+            self.assertTrue(errback.called)
+
+    def test_drain_nowait(self):
+        c = Connection(transport=Mock)
+        c.drain_events = Mock()
+        c.drain_events.side_effect = socket.timeout()
+
+        c.more_to_read = True
+        self.assertFalse(c.drain_nowait())
+        self.assertFalse(c.more_to_read)
+
+        c.drain_events.side_effect = socket.error()
+        c.drain_events.side_effect.errno = errno.EAGAIN
+        c.more_to_read = True
+        self.assertFalse(c.drain_nowait())
+        self.assertFalse(c.more_to_read)
+
+        c.drain_events.side_effect = socket.error()
+        c.drain_events.side_effect.errno = errno.EPERM
+        with self.assertRaises(socket.error):
+            c.drain_nowait()
+
+        c.more_to_read = False
+        c.drain_events = Mock()
+        self.assertTrue(c.drain_nowait())
+        c.drain_events.assert_called_with(timeout=0)
+        self.assertTrue(c.more_to_read)
+
+    def test_supports_heartbeats(self):
+        c = Connection(transport=Mock)
+        c.transport.supports_heartbeats = False
+        self.assertFalse(c.supports_heartbeats)
+
+    def test_is_evented(self):
+        c = Connection(transport=Mock)
+        c.transport.supports_ev = False
+        self.assertFalse(c.is_evented)
+
+    def test_eventmap(self):
+        c = Connection(transport=Mock)
+        c.transport.eventmap.return_value = {1: 1, 2: 2}
+        self.assertDictEqual(c.eventmap, {1: 1, 2: 2})
+        c.transport.eventmap.assert_called_with(c.connection)
+
+    def test_manager(self):
+        c = Connection(transport=Mock)
+        self.assertIs(c.manager, c.transport.manager)
+
+    def test_copy(self):
+        c = Connection('amqp://example.com')
+        self.assertEqual(copy(c).info(), c.info())
+
+    def test_switch(self):
+        c = Connection('amqp://foo')
+        c._closed = True
+        c.switch('redis://example.com//3')
+        self.assertFalse(c._closed)
+        self.assertEqual(c.hostname, 'example.com')
+        self.assertEqual(c.transport_cls, 'redis')
+        self.assertEqual(c.virtual_host, '/3')
+
+    def test_maybe_switch_next(self):
+        c = Connection('amqp://foo;redis://example.com//3')
+        c.maybe_switch_next()
+        self.assertFalse(c._closed)
+        self.assertEqual(c.hostname, 'example.com')
+        self.assertEqual(c.transport_cls, 'redis')
+        self.assertEqual(c.virtual_host, '/3')
+
+    def test_maybe_switch_next_no_cycle(self):
+        c = Connection('amqp://foo')
+        c.maybe_switch_next()
+        self.assertFalse(c._closed)
+        self.assertEqual(c.hostname, 'foo')
+        self.assertIn(c.transport_cls, ('librabbitmq', 'pyamqp'))
+
+    def test_heartbeat_check(self):
+        c = Connection(transport=Transport)
+        c.transport.heartbeat_check = Mock()
+        c.heartbeat_check(3)
+        c.transport.heartbeat_check.assert_called_with(c.connection, rate=3)
+
+    def test_completes_cycle_no_cycle(self):
+        c = Connection('amqp://')
+        self.assertTrue(c.completes_cycle(0))
+        self.assertTrue(c.completes_cycle(1))
+
+    def test_completes_cycle(self):
+        c = Connection('amqp://a;amqp://b;amqp://c')
+        self.assertFalse(c.completes_cycle(0))
+        self.assertFalse(c.completes_cycle(1))
+        self.assertTrue(c.completes_cycle(2))
+
     def test__enter____exit__(self):
         conn = self.conn
         context = conn.__enter__()
@@ -177,7 +331,7 @@ class test_Connection(TestCase):
             def close_connection(self, connection):
                 raise _CustomError('foo')
 
-        conn = BrokerConnection(transport=MyTransport)
+        conn = Connection(transport=MyTransport)
         conn.connect()
         conn.close()
         self.assertTrue(conn._closed)
@@ -190,7 +344,7 @@ class test_Connection(TestCase):
 
     def test_close_when_default_channel_close_raises(self):
 
-        class Conn(BrokerConnection):
+        class Conn(Connection):
 
             @property
             def connection_errors(self):
@@ -299,7 +453,7 @@ class test_Connection(TestCase):
         class MyTransport(Transport):
             channel_errors = (KeyError, ValueError)
 
-        conn = BrokerConnection(transport=MyTransport)
+        conn = Connection(transport=MyTransport)
         self.assertTupleEqual(conn.channel_errors, (KeyError, ValueError))
 
     def test_connection_errors(self):
@@ -307,7 +461,7 @@ class test_Connection(TestCase):
         class MyTransport(Transport):
             connection_errors = (KeyError, ValueError)
 
-        conn = BrokerConnection(transport=MyTransport)
+        conn = Connection(transport=MyTransport)
         self.assertTupleEqual(conn.connection_errors, (KeyError, ValueError))
 
 
@@ -316,8 +470,8 @@ class test_Connection_with_transport_options(TestCase):
     transport_options = {'pool_recycler': 3600, 'echo': True}
 
     def setUp(self):
-        self.conn = BrokerConnection(port=5672, transport=Transport,
-                                     transport_options=self.transport_options)
+        self.conn = Connection(port=5672, transport=Transport,
+                               transport_options=self.transport_options)
 
     def test_establish_connection(self):
         conn = self.conn
@@ -359,6 +513,18 @@ class ResourceCase(TestCase):
         [chan.release() for chan in chans]
         self.assertState(P, 10, 0)
 
+    def test_acquire_prepare_raises(self):
+        if self.abstract:
+            return
+        P = self.create_resource(10, 0)
+
+        self.assertEqual(len(P._resource.queue), 10)
+        P.prepare = Mock()
+        P.prepare.side_effect = IOError()
+        with self.assertRaises(IOError):
+            P.acquire(block=True)
+        self.assertEqual(len(P._resource.queue), 10)
+
     def test_acquire_no_limit(self):
         if self.abstract:
             return
@@ -433,8 +599,7 @@ class test_ConnectionPool(ResourceCase):
     abstract = False
 
     def create_resource(self, limit, preload):
-        return BrokerConnection(port=5672, transport=Transport) \
-                    .Pool(limit, preload)
+        return Connection(port=5672, transport=Transport).Pool(limit, preload)
 
     def test_setup(self):
         P = self.create_resource(10, 2)
@@ -443,6 +608,12 @@ class test_ConnectionPool(ResourceCase):
         self.assertIsNotNone(q[1]._connection)
         self.assertIsNone(q[2]()._connection)
 
+    def test_release_no__debug(self):
+        P = self.create_resource(10, 2)
+        R = Mock()
+        R._debug.side_effect = AttributeError()
+        P.release_resource(R)
+
     def test_setup_no_limit(self):
         P = self.create_resource(None, None)
         self.assertFalse(P._resource.queue)
@@ -450,7 +621,7 @@ class test_ConnectionPool(ResourceCase):
 
     def test_prepare_not_callable(self):
         P = self.create_resource(None, None)
-        conn = BrokerConnection('memory://')
+        conn = Connection('memory://')
         self.assertIs(P.prepare(conn), conn)
 
     def test_acquire_channel(self):
@@ -463,7 +634,7 @@ class test_ChannelPool(ResourceCase):
     abstract = False
 
     def create_resource(self, limit, preload):
-        return BrokerConnection(port=5672, transport=Transport) \
+        return Connection(port=5672, transport=Transport) \
                     .ChannelPool(limit, preload)
 
     def test_setup(self):
@@ -481,6 +652,6 @@ class test_ChannelPool(ResourceCase):
 
     def test_prepare_not_callable(self):
         P = self.create_resource(10, 0)
-        conn = BrokerConnection('memory://')
+        conn = Connection('memory://')
         chan = conn.default_channel
         self.assertIs(P.prepare(chan), chan)
