@@ -4,16 +4,15 @@ kombu.utils
 
 Internal utilities.
 
-:copyright: (c) 2009 - 2012 by Ask Solem.
-:license: BSD, see LICENSE for more details.
-
 """
 from __future__ import absolute_import
 
 import importlib
+import random
 import sys
 
 from contextlib import contextmanager
+from itertools import count, repeat
 from time import sleep
 from uuid import UUID, uuid4 as _uuid4, _uuid_generate_random
 
@@ -213,7 +212,7 @@ def retry_over_time(fun, catch, args=[], kwargs={}, errback=None,
     interval_range = fxrange(interval_start,
                              interval_max + interval_start,
                              interval_step, repeatlast=True)
-    for retries, interval in enumerate(interval_range):  # for infinity
+    for retries in count():
         try:
             return fun(*args, **kwargs)
         except catch, exc:
@@ -221,12 +220,12 @@ def retry_over_time(fun, catch, args=[], kwargs={}, errback=None,
                 raise
             if callback:
                 callback()
-            if errback:
-                errback(exc, interval)
-            for i in fxrange(stop=interval or 1.0):
-                if i and callback:
-                    callback()
-                sleep(i)
+            tts = errback(exc, interval_range, retries) if errback else None
+            if tts:
+                for i in range(int(tts / interval_step)):
+                    if callback:
+                        callback()
+                    sleep(interval_step)
 
 
 def emergency_dump_state(state, open_file=open, dump=None):
@@ -361,3 +360,35 @@ def nested(*managers):  # pragma: no cover
                 raise exc[0], exc[1], exc[2]
     finally:
         del(exc)
+
+
+def shufflecycle(it):
+    it = list(it)  # don't modify callers list
+    shuffle = random.shuffle
+    for _ in repeat(None):
+        shuffle(it)
+        yield it[0]
+
+
+def entrypoints(namespace):
+    try:
+        from pkg_resources import iter_entry_points
+    except ImportError:
+        return iter([])
+    return ((ep, ep.load()) for ep in iter_entry_points(namespace))
+
+
+class ChannelPromise(object):
+
+    def __init__(self, contract):
+        self.__contract__ = contract
+
+    def __call__(self):
+        try:
+            return self.__value__
+        except AttributeError:
+            value = self.__value__ = self.__contract__()
+            return value
+
+    def __repr__(self):
+        return '<promise: %r>' % (self(), )
